//
//  TVTableGenerator.m
//  TVPlugin
//
//  Created by Takaho A. Endo on 05/07/15.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#import "TVTableGenerator.h"


@implementation TVTableGenerator

-(TVTableGenerator*)initWithWebView: (WebView*)view {
	[ super init ];
	CSVFilename = nil;
	areaTable = nil;
	return self;
}

-(void)dealloc {
	if ( CSVFilename != nil ) {
		[ CSVFilename release ];
	}
	if ( areaTable != nil ) {
		[ areaTable release ];
	}
	[ super dealloc ];
}

-(void)windowScriptObjectAvailable:(WebScriptObject*)wso {
	[wso setValue:self forKey:@"TVTableGenerator" ];
}

+(NSString*)webScriptNameForSelector:(SEL)aSel {
	if ( aSel == @selector(setProgramFile:) ) 
		return @"setProgramFile";
	if ( aSel == @selector(getProgramListWithRange: channel: start: end:) ) 
		return @"getProgramListWithRange";
	if ( aSel == @selector(getCurrentPrograms:) ) 
		return @"getCurrentPrograms";
	if ( aSel == @selector(getChannelName:) ) 
		return @"getChannelName";
	if ( aSel == @selector(getChannelSize) ) 
		return @"getChannelSize";
	if ( aSel == @selector(getArea) )
		return @"getArea";
	if ( aSel == @selector(removeProgramFile:) )
		return @"removeProgramFile";
	if ( aSel == @selector(getProgramFile) )
		return @"getProgramFile";
	return nil;
}

+(BOOL)isSelectorExcludedFromWebScript:(SEL)aSel {
	if ( aSel == @selector(setProgramFile:) 
	|| aSel == @selector(removeProgramFile:)
	|| aSel == @selector(getProgramFile)
	|| aSel == @selector(getProgramListWithRange: channel: start: end:)
	|| aSel == @selector(getCurrentPrograms:)
	|| aSel == @selector(getChannelName:)
	|| aSel == @selector(getArea)
	|| aSel == @selector(getChannelSize) ) {
		return NO;
	}
	
	return YES;
}
	
+(BOOL)isKeyExcludedFromWebScript:(const char*)k {
	return YES;
}

-(void)setProgramFile: (NSString*)filename {
//	NSLog( @"%s %s %d: load %@", __FILE__, __func__, __LINE__, filename );
	NSFileManager* fm = [ NSFileManager defaultManager ];
	if ( [ fm fileExistsAtPath: filename ] == NO ) {
		NSLog( @"cannot find %@", filename );
		return;
	}
	if ( CSVFilename != nil ) {
		[ self->CSVFilename release ];
	}
	self->CSVFilename = filename;
	[ self->CSVFilename retain ];
	if ( areaTable != nil ) {
		[ self->areaTable release ];
	}
	self->areaTable = [ AreaProgram areaProgramWithFile : filename ];
	[ self->areaTable retain ];
}

-(void)removeProgramFile: (NSString*)filename {
	NSFileManager* fm = [ NSFileManager defaultManager ];
	if ( [ fm fileExistsAtPath: filename ] ) {
		NSLog( @"removing %@", filename  );
		[ fm removeFileAtPath: filename handler: nil ];
	} else {
		NSLog( @"%@ does not exist", filename );
	}
}

-(NSString*)getProgramFile {
	if ( CSVFilename == nil ) return @"";
	NSFileManager* fm = [ NSFileManager defaultManager ];
	if ( [ fm fileExistsAtPath: CSVFilename ] ) {
		return CSVFilename;
	}
	return @"";
}

-(NSString*)getProgramListWithRange : (NSString*) format channel:(int)number start:(int)startMinutes	end:(int)endMinutes {
	if ( areaTable == nil ) return @"";
	ProgramTable* table = [ areaTable getChannelAt : number ];
	if ( table == nil ) {
		NSLog( @"out of range" );
		return @"";
	}
	return [ table toHTML : format start: startMinutes end: endMinutes ];
}

-(NSString*)getCurrentPrograms: (int) minutes {
	if ( self->areaTable == nil ) {
		NSLog( @"no table" );
		return @"";
	}
	NSString* progs = [ areaTable toStringProgramList: minutes ];
	//NSLog( @"\n%@", progs );
	return progs;
}

-(NSString*)getArea {
	if ( areaTable == nil ) return @"";
	return [ areaTable getArea ];
}

-(NSString*)getChannelName : (int)num {
	if ( areaTable == nil ) return @"";
	ProgramTable* channel = [ areaTable getChannelAt : num ];
	if ( channel == nil ) {
//		NSLog( @"out of range" ); 
		return @"";
	}
	return [ channel getChannelName ];
}

-(int)getChannelSize {
	if ( areaTable == nil ) return 0;
	return [ areaTable getChannelSize ];
}

@end
