#!/usr/bin/python
# -*- coding:utf-8 -*-

import os, string, re, sys

class Program:
    GENRE_NONE   = 0
    GENRE_FOOD   = 1
    GENRE_DRAMA  = 2
    GENRE_SPORTS = 3
    GENRE_ANIME  = 4
    GENRE_MUSIC  = 5
    GENRE_MOVIE  = 6
        
    regex_time = re.compile( "<b>(\\d\\d:\\d\\d)</b>" )
    regex_title = re.compile( "<a[^>]+href=([^>\\s]+)\\s*>(.*?)</a>" )
    regex_rmtag = re.compile( "<.*?>" )
    regex_img   = re.compile( "<img\\s.*?alt=['\"]?(.*?)['\"]?[\\s>][^>]*>" )

    def __init__( self, data, span ):
        data = data.strip()
        self.comment = ""
        self.genre = self.GENRE_NONE
        #print data
        mt = self.regex_time.search( data )
        self.time = mt.group( 1 )
        hr, mn = self.time.split( ':' )
        #print hr, mn
        self.startTime = string.atoi( hr ) * 60 + string.atoi( mn )
        self.endTime = self.startTime + span

        ml = self.regex_title.search( data, mt.end() )
        #print ml
        self.title = ml.group( 2 )
        self.link = ml.group( 1 )

        mi = self.regex_img.search( self.title )
        if mi:
            while mi:
                #print self.title
                #print mi.group( 1 ), mi.start(), mi.end()
                alt = mi.group( 1 )
                if alt == 'ニュース'   : alt = '[Ｎ]'
                elif alt == '天気予報' : alt = '[天]'
                elif alt == '再放送'   : alt = '[再]'
                elif alt == '映画'   :
                    alt = ''
                    self.genre = self.GENRE_MOVIE
                    pass
                elif alt == '最終回':
                    alt = ''
                    self.comment = alt
                    pass
                elif alt == '前編' or alt == '中編' or alt == '後編':
                    self.comment = alt
                    alt = ''
                    pass
                self.title = self.title[ 0: mi.start() ] + alt + self.title[ mi.end(): ]
                #self.regex_img.sub( "\[\\1\]", self.title )
                #print self.title
                mi = self.regex_img.search( self.title )
                pass
           # sys.exit()

        self.description = self.regex_rmtag.sub( "", data[ ml.end(): ] ).strip()
        self.span = span
        self.idvalue = Program.counter
        Program.counter += 1
        pass

    counter = 0
    
    def setID( self, idnum ):
        self.idvalue = idnum
        pass
    def getID( self ):
        return self.idvalue
    
    def setGenre( self, genre ):
        self.genre = genre
        pass

    def setSpan( self, span ):
        if span < 0: span += 1440
        self.span = span
        self.endTime = self.startTime + span
        pass

    def offsetTime( self, minutes ):
        if self.startTime < 1440:
            self.startTime += minutes
            pass
        if self.endTime < 1440:
            self.endTime += minutes
            pass
        pass
    

    def __repr__( self ):
        output = "%d,%d,%s,%s,%s,%d" % ( self.startTime, self.span, self.title, self.description, self.comment, self.genre )
        return output
    pass

# class RadioProgram ( Program ):
#     regex_time = re.compile( "<small>\\s*<b>(\\d\\d:\\d\\d)</b>\\s*</small>" )
#     regex_title = re.compile( "<small>(.*?)</small>" )
#     regex_img   = re.compile( "<img\\s.*?alt=['\"]?(.*?)['\"]?[\\s>][^>]*>" )
#     def __init__( self, data, span ):
#         data = data.stip()

#         self.comment = ""
#         self.genre = 0

#         mt = self.regex_time.search( data )
#         self.time = mt.group( 1 )
#         hr, mn = self.time.split( ':' )
#         #print hr, mn
#         self.startTime = string.atoi( hr ) * 60 + string.atoi( mn )
#         self.endTime = self.startTime + span

#         ml = self.regex_title.search( data, mt.end() )
#         #print ml
#         self.title = ml.group( 1 )
#         #self.link = ml.group( 1 )

#         self.description = ""
#         self.span = span
#         pass
#     pass

class ProgramTable:
    def __init__( self, progs = None ):
        self.name = ""
        if progs == None:
            self.programs = []
        else:
            self.programs = progs
        pass

    def set_name( self, name ):
        self.name = name
        pass

    def get_name( self ):
        return self.name

    def append( self, prog ):
        self.programs.append( prog )
        pass

    def find_program_at( self, time ):
        for p in self.programs:
            if p.startTime == time:
                return p
            elif p.startTime > time:
                return None
            pass
        return None

    def is_left( self, table ):
        for p in self.programs:
            q = table.find_program_at( p.startTime )
            if q:
                if p.getID() < q.getID(): return True
                return False
            pass
        return False
    pass

if __name__ == "__main__":
    import urllib, tempfile, time
    if len( sys.argv ) <= 1:
        raise "No options"
    i = 1

    use_urllib = False
    file_out = sys.stdout
    filename_output = None
    flag_radio = False
    area = 'tokyo'
    wave = 'vhf'
    while i < len( sys.argv ) - 1:
        ar = sys.argv[ i ]
        if ar == '-o':
            filename_output = sys.argv[ i + 1 ]
            pass
        elif ar == '-area':
            area = sys.argv[ i + 1 ]
        elif ar == '-wave':
            wave = sys.argv[ i + 1 ]
            pass
        i += 1
        pass
    
    if filename_output:
        file_out = open( filename_output, "w" )
        pass

    if wave == 'uhf' or wave == 'bs':
        baseurl = "http://tv.yahoo.co.jp/uhf_bs/%s/%s24.html?g="
    elif wave == 'bs_digital':
        baseurl = "http://tv.yahoo.co.jp/bs_digital/%s/%s24.html?g="
    elif wave == 'am':
        flag_radio = True
        baseurl = "http://tv.yahoo.co.jp/radio/%s/%s24.html"
    elif wave == 'fm':
        flag_radio = True
        baseurl = "http://tv.yahoo.co.jp/radio/%s/%s24fm.html"
    else:
        baseurl = url = "http://tv.yahoo.co.jp/vhf/%s/%s24.html?g="
        pass

    lt = time.localtime()
    today = "%d%s%s" % ( lt[ 0 ], ( "0%d" % lt[ 1 ] )[ -2: ], ( "0%d" % lt[ 2 ] )[ -2: ] )
    url = baseurl % ( area, today )

    if use_urllib:
        stream = urllib.urlopen( url )
        fn_tmp = tempfile.mktemp()
        open( fn_tmp, "w" ).write( stream.read() )
        stream.close()
    else:
        fn_tmp = tempfile.mktemp()
        command = "/usr/bin/curl \"%s\" --silent -o %s" % ( url, fn_tmp )
        #print command
        os.system( command )
        pass
    
    fn_utf = tempfile.mktemp()
    os.system( "/usr/bin/iconv -f euc-jp -t utf-8 %s > %s" % ( fn_tmp, fn_utf ) )
    os.unlink( fn_tmp )

    # execute external script for radio programs
    if flag_radio:
        import radio
        radio.convert_radio_html_to_csv( fn_utf, file_out )
        file_out.close()
        os.unlink( fn_utf )
        sys.exit()

    
    fn = open( fn_utf )
    line = fn.readline()
    area = ""
    wave = ""
    date = ""
    # read title
    re_title = re.compile( "<title>(.*?)</title>" )
    while line != "":
        m = re_title.search( line )
        if m:
            title = m.group( 1 )
            area = title
            m = re.search( "\[(.*?)/(.*?)/[^\]]+\]", title )
            if m:
                area = m.group( 2 )
                wave = m.group( 1 )
                pass
            m = re.search( "(\\d+)月(\\d+)日", title )
            if m:
                date = m.group( 1 ) + "/" + m.group( 2 )
                pass
            break
        line = fn.readline()
        pass
    
    # read channel names
    channels = []
    channelurls = {}
    flag_start = False
    r_channel = re.compile( "<font color=''>(.*?)</font>" )
    r_link = re.compile( "href='(.*?)'" )
    while line != "":
        line = line.strip()
        if line == "<!--- navi-->": flag_start = True
        if flag_start:
            #print line
            if line.find( "</tr>" ) >= 0: break
            m_channel = r_channel.search( line )
            if m_channel:
                cn = re.sub( "<.*?>", "", m_channel.group( 1 ) )
                channels.append( cn )
                #print cn
                m_link = r_link.search( line )
                if m_link:
                    channelurls[ cn ] = m_link.group( 1 )
                    #print channelurls[ cn ]
                    pass
                pass
            pass
        line = fn.readline()
        pass
    #sys.exit()
    
    # read programs
    programs = {}
    storedorder = []

    r_span = re.compile( "rowspan='?(\\d+)'?" )
    r_bgcol = re.compile( "bgcolor='(#[\0-9a-zA-Z]+)'" )
    r_prog = re.compile( "href=/bin/search\?id=(\\d+)" )
    #r_radio = re.compile( "<td><small><b>\\d\\d:\\d\\d</b></small>.*?</td></tr>" )
    flag_midnight = False

    genre = Program.GENRE_NONE
    while line != "":
        if line.find( "<small>0</small>" ) >= 0:
            flag_midnight = True
        m_span = r_span.search( line )
        if m_span:
            span = string.atoi( m_span.group( 1 ) )
            m_bgcol = r_bgcol.search( line )
            if m_bgcol: # Genre
                bgcol = m_bgcol.group( 1 )
                if bgcol != '#FFFFFF': print bgcol
                if   bgcol == '#ffcf9c': genre = Program.GENRE_FOOD
                elif bgcol == '#ffffce': genre = Program.GENRE_DRAMA
                elif bgcol == '#ddeeff': genre = Program.GENRE_ANIME
                elif bgcol == '#eeddff': genre = Program.GENRE_SPORTS
                elif bgcol == '#ddffdd': genre = Program.GENRE_MUSIC
                elif bgcol == '#ffdddd': genre = Program.GENRE_MOVIE
                else: genre = Program.GENRE_NONE
            else:
                genre = Program.GENRE_NONE
                pass
            pass
#         if flag_radio:
#             m_radio = r_radio.match( line )
#             if m_radio:
#                 p = RadioProgram( line, span )
#             if flag_midnight:
#                 p.offsetTime( 1440 )
#                 pass
#             idval = ""#string.atoi( m_prog.group( 1 ) )
#             #print idval, p
#             programs[ idval ] = p
#             storedorder.append( p )
#             pass
#         else:
        m_prog = r_prog.search( line )
        if m_prog:
            p = Program( line, span )
            if genre != Program.GENRE_NONE:
                p.setGenre( genre )
                pass
            if flag_midnight:
                p.offsetTime( 1440 )
                pass
            idval = string.atoi( m_prog.group( 1 ) )
            #print idval, p
            programs[ idval ] = p
            #p.setID( idval )
            pass
        line = fn.readline()
        pass
    fn.close()
    
    keys = programs.keys()
    keys.sort()
    delims = []
    delims.append( 0 )
    for i in range( 1, len( keys ) ):
        p0 = programs[ keys[ i - 1 ] ]
        p1 = programs[ keys[ i ] ]
        t0 = p0.endTime
        t1 = p1.startTime
        if p0.startTime > t1:
            t1 += 1440
            pass
        if p0.startTime > p1.startTime:
            #if p0.description.find( '終了' ) >= 0 \
            #   or ( t0 != t1 and ( p0.startTime < 300 and p0.endTime > 300 ) ):
            delims.append( i )
            #print p0
            #print "-----------------"
            #print p1
            #print
            pass
        elif t0 != t1:
            #print ">>>", t0, t1, "St0 ", p0.startTime, ", St1 ", p1.startTime
            #print p0
            p0.setSpan( p1.startTime - p0.startTime )
            #print p0
            #print 
            pass
        #print p0
        pass
    delims.append( len( keys ) )

    tables = []
    for d in range( 0, len( delims ) - 1 ):
        table = ProgramTable()
        for key in keys[ delims[ d ]: delims[ d + 1 ] ]:
            table.append( programs[ key ] )
            pass
        tables.append( table )
        pass

    for i in range( len( tables ) ):
        table_left = tables[ i ]
        pos_left = i
        for j in range( i + 1, len( tables ) ):
            if tables[ j ].is_left( table_left ):
                table_left = tables[ j ]
                pos_left = j
                pass
            pass
        #print pos_left
        tables[ pos_left ] = tables[ i ]
        tables[ i ] = table_left
        table_left.set_name( channels[ i ] )
        pass
    #print tables

    file_out.write( "#%s,%s,%s\n" % ( area, date,wave ) )
    for i in range( len( tables ) ):
        table = tables[ i ]
        ch = channels[ i ]
        if channelurls.has_key( ch ):
            file_out.write( "*%s,%s\n" % ( ch, channelurls[ ch ] ) )
        else:
            file_out.write( "*%s\n" % ch )
            pass
        for p in table.programs:
            file_out.write( "%s\n" % repr( p ) )
            pass
        pass
    file_out.close()
    os.unlink( fn_utf )
    pass

